
// $Id: InternalObjectsFactory.java,v 1.1 1999/03/22 09:26:50 larsga Exp $

package XSA;

import java.util.Hashtable;
import org.xml.sax.*;
import org.xml.sax.helpers.*;

/**
 * This class is used by the XSA engine to create objects it uses
 * internally.  It can be subclassed by advanced applications that
 * need to change the internal workings of the engine.
 */

public class InternalObjectsFactory {
  
  /**
   * Creates a SAX parser.
   * @param parserDriver The driver to instantiate. If null the factory will
   *                     try using the org.xml.sax.parser property instead.
   * @exception InternalXSAException Thrown with an appropriate error message
   *                                 if the parser cannot be created.
   */

  public Parser createParser(String parserDriver) throws InternalXSAException {
    try {
      if (parserDriver==null) 
	return ParserFactory.makeParser();
      else
	return ParserFactory.makeParser(parserDriver);
    }
    catch (NullPointerException e) {
      throw new InternalXSAException("No parser specified. Set the "+
				      "org.xml.sax.parser property or "+
				      "specify one.");
    }
    catch (ClassNotFoundException e) {
      throw new InternalXSAException("The specified SAX parser was not "+
				     "found.");
    }
    catch (InstantiationException e) {
      throw new InternalXSAException("Couldn't instantiate SAX parser.");
    }
    catch (IllegalAccessException e) {
      throw new InternalXSAException("Didn't have permission to access SAX "+
				     "parser.");
    }
  }

  /**
   * Sets up and creates the XSADocumentHandler object.
   * @param parser The SAX parser that will use the detector.
   * @param client The XSA client that receives events.
   * @param url The URL of the document to be parsed.
   * @param pids The product IDs of the products to check.
   * @param factory The object factory to be used by the SAX handlers.
   */

  public XSADocumentHandler createDocHandler(Parser parser,
					     XSAClient client,
					     String url,
					     Hashtable pids,
					     ObjectFactory factory) {
    Hashtable hash=new Hashtable();
    hash.put("SOFTPKG",new OSDSAXHandler(client,url,pids,factory));
    hash.put("xsa",new XSASAXHandler(client,url,pids,factory));
    return new SAXDoctypeDetector(parser,hash);
  }

  /**
   * Creates the error handler to be used during parsing.
   * @param url The URL of the document to be parsed.
   * @param client The XSA client that receives events.
   */

  public ErrorHandler createErrorHandler(String url, XSAClient client) {
    return new SAXErrorHandler(url,client);
  }
  
}
