
package XSA;

import java.util.Hashtable;
import java.util.Vector;
import java.util.Enumeration;

import org.xml.sax.*;

/**
 * This class chooses between different DocumentHandlers based on the root
 * element type of the parsed document.
 */

class SAXDoctypeDetector extends HandlerBase implements XSADocumentHandler {
  private Hashtable handlers;
  private Parser    driver;
  private Vector    pis;
  private XSADocumentHandler chosenHandler;

  private Locator   loc;

  /**
   * Creates the detector.
   * @param _handlers A hashtable that maps from root element type to the
   *                  DocumentHandler to use.
   * @param _driver   The SAX driver.
   */

  public SAXDoctypeDetector(Parser _driver, Hashtable _handlers) {
    handlers=_handlers;
    driver=_driver;
    pis=new Vector();
    loc=null;
  }

  // === XSADocumentHandler methods ===

  public Vendor getVendor() {
    return chosenHandler.getVendor();
  }
  
  // === Handlerbase methods ===
  
  public void setDocumentLocator(Locator locator) {
    loc=locator;
  }

  public void processingInstruction(String target, String data) {
    pis.addElement(target+" "+data);
  }

  public void startElement(String name, AttributeList attrs)
       throws SAXException{
    DocumentHandler handler=(DocumentHandler) handlers.get(name);
    if (handler==null) 
      throw new SAXException("Unknown root element '"+name+
			     "', cannot handle this document.");

    driver.setDocumentHandler(handler); 
    handler.startDocument();

    Enumeration enum=pis.elements();
    while(enum.hasMoreElements()) {
      String pi=(String) enum.nextElement();
      int pos=pi.indexOf(" ");

      handler.processingInstruction(pi.substring(0,pos),pi.substring(pos+1));
    }

    if (loc!=null) handler.setDocumentLocator(loc);
    handler.startElement(name,attrs);
    chosenHandler=(XSADocumentHandler) handler;
  }
  
}
